package com.infosupport.jewelshop;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.image.ContextImage;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.link.PopupSettings;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import org.apache.wicket.util.string.UrlUtils;

import com.infosupport.jewelshop.domain.Jewel;
import com.infosupport.jewelshop.domain.JewelNotFoundException;
import com.infosupport.jewelshop.srv.JewelInventorySrv;

public class JewelDetailsPage extends MasterPage {

	public static final String PAGEPARAM_JEWELID = "jewelId";

	public static PageParameters createPageParameters(Jewel jewel) {
		PageParameters params = new PageParameters();
		params.set(JewelDetailsPage.PAGEPARAM_JEWELID, jewel.getId());
		return params;
	}

	private static final long serialVersionUID = -2211130047529646727L;

	public JewelDetailsPage(PageParameters parameters)
			throws JewelNotFoundException {
		super(parameters);
		Jewel jewel = getJewelByPageParam(parameters);
		setDefaultModel(new CompoundPropertyModel<Jewel>(jewel));
		add(new Label("nameTitle", Model.of(jewel.getName())));
		add(new Label(Jewel.PROP_NAME));
		add(new Label(Jewel.PROP_PRICE));
		add(new Label(Jewel.PROP_DETAILS));

		String imageUrl = "images/" + jewel.getImage();
		add(new ContextImage("image", imageUrl));
		add(createPictureLink(imageUrl));
		add(createOrderNowLink(jewel));
	}

	private Jewel getJewelByPageParam(PageParameters parameters)
			throws JewelNotFoundException {
		StringValue stringValue = parameters.get(PAGEPARAM_JEWELID);
		Long jewelId = stringValue.toLongObject();

		Jewel jewel = JewelInventorySrv.getInstance().getJewelById(jewelId);
		return jewel;
	}

	private AjaxLink<Jewel> createOrderNowLink(Jewel jewel) {
		AjaxLink<Jewel> orderNow = new AjaxLink<Jewel>("oder_now",
				Model.of(jewel)) {
			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target) {
				onClickOrderNowLink(target, getModelObject());
			}

		};
		return orderNow;
	}

	private ExternalLink createPictureLink(String imageUrl) {
		String fullImageUrl = UrlUtils.rewriteToContextRelative(imageUrl,
				RequestCycle.get());
		ExternalLink pictureLink = new ExternalLink("bigPicutereLink",
				fullImageUrl);
		pictureLink.setPopupSettings(new PopupSettings("Jewel Image", 0)
				.setHeight(400).setWidth(400));
		return pictureLink;
	}

	public JewelDetailsPage(Jewel jewel) {
		super(new PageParameters());
	}

	private void onClickOrderNowLink(AjaxRequestTarget target, Jewel jewel) {
		JewelDetailsPage.this.getSession().getBasket().addItem(jewel, 1);
		target.add(get(BASKET_PANEL_ID));
	}

}
