package com.infosupport.jewelshop;

import java.lang.reflect.InvocationTargetException;

import org.apache.wicket.ConverterLocator;
import org.apache.wicket.IConverterLocator;
import org.apache.wicket.Page;
import org.apache.wicket.Session;
import org.apache.wicket.WicketRuntimeException;
import org.apache.wicket.protocol.http.WebApplication;
import org.apache.wicket.request.IRequestHandler;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.Response;
import org.apache.wicket.request.cycle.AbstractRequestCycleListener;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.handler.PageProvider;
import org.apache.wicket.request.handler.RenderPageRequestHandler;

import com.infosupport.jewelshop.domain.DutchZipCode;
import com.infosupport.jewelshop.domain.DutchZipCodeConverter;
import com.infosupport.jewelshop.domain.JewelNotFoundException;

/**
 * Application object for your web application. If you want to run this
 * application without deploying, run the Start class.
 * 
 * @see com.infosupport.jewelshop.Start#main(String[])
 */
public class WicketApplication extends WebApplication {

	@Override
	public Class<? extends Page> getHomePage() {
		return HomePage.class;
	}

	/**
	 * @see org.apache.wicket.Application#init()
	 */
	@Override
	public void init() {
		super.init();
		mountPage("/home", HomePage.class);
		mountPage("/details", JewelDetailsPage.class);
		mountPage("/contact", ContactPage.class);
		mountPage("/products", JewelsOverview.class);
		mountPage("/error", ErrorPage.class);

		getRequestCycleListeners().add(new AbstractRequestCycleListener() {

			public IRequestHandler onException(RequestCycle cycle, Exception ex) {
				Throwable cause = ex;
				if (ex instanceof WicketRuntimeException) {
					WicketRuntimeException wre = (WicketRuntimeException) ex;
					cause = wre.getCause();
				}
				if (cause instanceof InvocationTargetException) {
					cause = ((InvocationTargetException) cause).getCause();
				}
				if (cause instanceof JewelNotFoundException) {
					return new RenderPageRequestHandler(new PageProvider(
							new ErrorPage()));
				}
				return null;
			}
		});
	}

	@Override
	public Session newSession(Request request, Response response) {
		return new JewelSession(request);
	}

	@Override
	protected IConverterLocator newConverterLocator() {
		ConverterLocator converterLocator = new ConverterLocator();
		converterLocator.set(DutchZipCode.class, new DutchZipCodeConverter());
		return converterLocator;
	}
}
