package com.infosupport.jewelshop.datatables;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

import com.infosupport.jewelshop.domain.Jewel;
import com.infosupport.jewelshop.srv.JewelInventorySrv;

public class JewelDataProvider extends SortableDataProvider<Jewel> {

	private static final long serialVersionUID = -5069503953535079655L;

	private transient List<Jewel> jewels;

	private final Comparator<Jewel> comparator = new JewelComparator();

	public JewelDataProvider() {
		setSort(Jewel.PROP_NAME, SortOrder.ASCENDING);
	}

	private void atach() {
		jewels = new ArrayList<Jewel>(JewelInventorySrv.getInstance()
				.getJewels());
	}

	public Iterator<? extends Jewel> iterator(int first, int count) {
		validateAtach();
		Collections.sort(jewels, comparator);
		if (first > jewels.size()) {
			first = 0;
		}
		if (first + count > jewels.size()) {
			return jewels.subList(first, jewels.size()).iterator();
		} else {
			return jewels.subList(first, first + count).iterator();
		}
	}

	public int size() {
		validateAtach();
		return jewels.size();
	}

	private void validateAtach() {
		if (isDetach()) {
			atach();
		}
	}

	public IModel<Jewel> model(Jewel object) {
		return Model.of(object);
	}

	private boolean isDetach() {
		return jewels == null;
	}

	@Override
	public void detach() {
		System.out.println("Detach");
		super.detach();
		jewels = null;
	}

	private class JewelComparator implements Comparator<Jewel>, Serializable {

		private static final long serialVersionUID = 9129738802172534721L;

		public int compare(Jewel o1, Jewel o2) {
			SortParam sort = getSort();
			int result;
			if (sort.getProperty().equals(Jewel.PROP_NAME)) {
				result = o1.getName().compareTo(o2.getName());
			} else {
				result = o1.getPrice().compareTo(o2.getPrice());
			}

			if (sort.isAscending()) {
				result = -result;
			}
			return result;
		}

	}
}
