package com.infosupport.jewelshop.forms;

import java.util.Arrays;
import java.util.List;

import org.apache.wicket.markup.html.form.AbstractChoice;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.CheckBoxMultipleChoice;
import org.apache.wicket.markup.html.form.CheckboxMultipleChoiceSelector;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RadioChoice;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.CompoundPropertyModel;

import com.infosupport.jewelshop.domain.Contact;
import com.infosupport.jewelshop.domain.Contact.ContactType;
import com.infosupport.jewelshop.domain.Country;

public class ContactForm extends Form<Contact> {

	private static final long serialVersionUID = -6724066842027297886L;

	public ContactForm(String id, Contact contact) {
		super(id, new CompoundPropertyModel<Contact>(contact));

		add(new TextField<String>(Contact.PROP_FIRSTNAME).setRequired(true));
		add(new TextField<String>(Contact.PROP_LASTNAME).setRequired(true));
		add(new TextField<String>(Contact.PROP_EMAIL).setRequired(true));
		add(new TextField<String>(Contact.PROP_BIRTHDATE));
		add(new TextField<String>(Contact.PROP_PHONE));
		add(generateCountryDropDown());
		add(new TextArea<String>(Contact.PROP_MESSAGE));
		CheckBoxMultipleChoice<String> interests = generateInterestOptionsChoice();
		add(interests);
		add(new CheckboxMultipleChoiceSelector("selectAll", interests));
		add(new RadioChoice<ContactType>(Contact.PROP_CONTACT_TYPE,
				Arrays.asList(ContactType.values()))
				.setChoiceRenderer(new EnumChoiceRenderer<ContactType>(this)));
		add(new CheckBox(Contact.PROP_CONTACT_BY_PHONE));
		add(new CheckBox(Contact.PROP_NEWSLETTER));
		add(new Button("submit"));
		add(new Button("cancel"));
	}

	private CheckBoxMultipleChoice<String> generateInterestOptionsChoice() {
		List<String> interestedOptions = Arrays.asList("Edelstenen", "Juwelen",
				"Oud goud");
		CheckBoxMultipleChoice<String> interests = new CheckBoxMultipleChoice<String>(
				Contact.PROP_INTERESTS, interestedOptions);
		return interests;
	}

	private AbstractChoice<Country, Country> generateCountryDropDown() {
		ChoiceRenderer<Country> countryRenderer = new ChoiceRenderer<Country>(
				Country.PROP_FULL_NAME, Country.PROP_COUNTRY_CODE);
		return new DropDownChoice<Country>(Contact.PROP_COUNTRY,
				Country.getPoplulairCountries())
				.setChoiceRenderer(countryRenderer);
	}

}
